/**
 * @author Frithjof Schaefer
 * @version 1.0
 */
package mrnice.android.apps;

import java.util.List;

import android.app.Activity;
import android.content.Context;
import android.hardware.*;
import android.os.Bundle;
import android.view.MotionEvent;
import android.widget.*;

//Diese Klasse implementiert das SensorEventListener Interface, und kann so auf
//verfgbare Sensoren und deren Werte bzw Wertnderungen reagieren
public class PlayGroundActivityOne extends Activity implements SensorEventListener {
    /** Called when the activity is first created. */
	
	//Um in eine neue Zeile zu springen, das sonst gelufige "\n" funzt bei mir(android?) nicht
	public final static String newLine = System.getProperty("line.separator");
	
	//Sensor Manager enthlt alle verfgbaren Sensoren
	public SensorManager sensorMangager;
	
	//Touchscreenanalyse, abgeleiteter Button siehe private Klasse ganz unten
	public ButtonWithMotionEvent button;
	
	//hnlich wie ein static div in HTML
	private LinearLayout sensorInfoPanel;
	
	//Damit der overflow gescroll werden kann
	private ScrollView scroll;
	
	//Textfelder fr die Sensoreninfos
	private TextView accText;
	private TextView magText;
	private TextView tempText;
	private TextView oriText;
	
	
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);       
        
        
        //Initialisierung des Sensormanagers
        sensorMangager = (SensorManager) getSystemService(SENSOR_SERVICE);
                      
        //Ab hier wird die UI zusammengebastelt      
        
        //Initialisieren der Textboxen
        accText = new TextView(this);
        magText = new TextView(this);
        tempText = new TextView(this);
        oriText = new TextView(this);

        //Den MotionButton einstellen
        button = new ButtonWithMotionEvent(this);      
        button.setTextSize(10.0F);
        button.setText("Touch me");
        button.setHeight(300);
        
        //Die Anzeigeliste fllen
        sensorInfoPanel = new LinearLayout(this);
        sensorInfoPanel.setOrientation(LinearLayout.VERTICAL);
        sensorInfoPanel.addView(button);
        sensorInfoPanel.addView(accText);
        sensorInfoPanel.addView(magText);
        sensorInfoPanel.addView(oriText);
        sensorInfoPanel.addView(tempText);        
        
        //Die Anzeigeliste ans Scrollview hngen (Ich habe gelesen, es sei das beste, wenn das ScrollView mglist nur ein Childview hat...)
        scroll = new ScrollView(this);
        scroll.addView(sensorInfoPanel);
        
        //Den Scrollbaren Inhalt anzeigen
        setContentView(scroll);
        
    }
    
    //So kann mann erstmal schauen, welche Sensoren so registriert sind
    private String getSensorNames(){
    	if(sensorMangager == null)
    		return "SensorManager is not initialized";         
        
        List<Sensor> availableSensors = sensorMangager.getSensorList(Sensor.TYPE_ALL);
        
        
        if(availableSensors.size() == 0)
        	return "There are no sensors available on your device";
        
        String out = "";
        for(Sensor aSensor:availableSensors){
        	out += aSensor.getName() + " Hersteller: " + aSensor.getVendor() + newLine;        	
        } 
        
        return out;
    
    }
   
    //Hier werden die Sensordaten bei einem gefeuerten Sensorevent in die Textboxen geschrieben. (Das passiert mehrmals die Sekunde, siehe "onResume() weiter unten"
	@Override
	public void onSensorChanged(SensorEvent event) {
		if(event.sensor.getType() == Sensor.TYPE_ACCELEROMETER){ //Wie schnell wird mein Handy im 3D-Koordinatensystem beschleunigt? 
			accText.setText("Acceleration: " +event.sensor.getName() +newLine +
							"X:" + Float.toString(event.values[0]) +"m/s^2" +newLine +
							"Y:" + Float.toString(event.values[1]) +"m/s^2" +newLine +
							"Z:" + Float.toString(event.values[2]) +"m/s^2" +newLine
			);
		
		}
		
		else if(event.sensor.getType() == Sensor.TYPE_MAGNETIC_FIELD){ //Wie wirkt ein magnetisches Feld auf mein Handy im 3D-Koordinatensystem? 
			magText.setText("Magnatic: " + event.sensor.getName() +newLine +
							"X:" + Float.toString(event.values[0]) + newLine +
							"Y:" + Float.toString(event.values[1]) + newLine +
							"Z:" + Float.toString(event.values[2]) + newLine
			);
		
		}
		
		else if(event.sensor.getType() == Sensor.TYPE_ORIENTATION){ // Wie ist die Ausrichtung(Kompass), und die horizontale und vertikale Lage?
			oriText.setText("Orientation: " + event.sensor.getName() +newLine +
					"X:" + Float.toString(event.values[0]) + newLine + //Kompass
					"Y:" + Float.toString(event.values[1]) + newLine + //Anzeige fr "wie im Flugzeug"
					"Z:" + Float.toString(event.values[2]) + newLine   //Anzeigedaten fr "wie im Flugzeug"
			);
		
		}
		
		else if(event.sensor.getType() == Sensor.TYPE_TEMPERATURE){ // Wie warm is mein Handy? Keine Auentemperatur versteht sich... evt wenn mans im Auto ausm Fenster hlt :-)
			tempText.setText("Temperature: " + event.sensor.getName() +newLine +
							"X:" + Float.toString(event.values[0]) + newLine
			);
		
		}	
		
		
	}
	
	//Je nach Anwendung, kann man auch so die Sensordaten abfangen
	@Override
	public void onAccuracyChanged(Sensor sensor, int accuracy) {
		// TODO Auto-generated method stub
		
	}
	
	//Wichtig!!! Die Registrierung muss hier erfolgen, sonst knallts!
	@Override
    protected void onResume() {
        super.onResume();        
        sensorMangager.registerListener(this,sensorMangager.getSensorList(Sensor.TYPE_ACCELEROMETER).get(0),SensorManager.SENSOR_DELAY_NORMAL );
        sensorMangager.registerListener(this,sensorMangager.getSensorList(Sensor.TYPE_MAGNETIC_FIELD).get(0),SensorManager.SENSOR_DELAY_NORMAL );
        sensorMangager.registerListener(this,sensorMangager.getSensorList(Sensor.TYPE_TEMPERATURE).get(0),SensorManager.SENSOR_DELAY_NORMAL );
        sensorMangager.registerListener(this,sensorMangager.getSensorList(Sensor.TYPE_ORIENTATION).get(0),SensorManager.SENSOR_DELAY_NORMAL );
        
        
        
	}
	
	//Wichtig!!! Sobald die Anwendung durch irgendwas (standby, display lock, etc.) gestoppt wird, mssen die Sensoren wieder freigegeben werden!
    @Override
    protected void onStop() {
        // unregister listener
       sensorMangager.unregisterListener(this);
        super.onStop();
    }    

    
    //###################################################
    //Private Klasse fr den Touchscreen "Sensor"
    private class ButtonWithMotionEvent extends Button {

    	
    	public ButtonWithMotionEvent(Context context) {
    		super(context);    		
    	}

    	//Hier nun eine eigene Touchevent-Implementierung, es wird immer aufgerufen, sobald
    	//der Button berhrt wird. Wenn der ScrollView aber scrollen will, geht die Kontrolle 
    	//verloren.... Kommt aber wieder ;)
    	@Override
    	public boolean onTouchEvent(MotionEvent event) {
    		//bool touched = super.onTouchEvent(event);
    		
    		//Eckenerkennung:
    		int edgeID = event.getEdgeFlags(); //Eckenerkennung funkioniert bei mir nicht...(Samsung Galaxy I7500)
    		String edge = "No Edge";
    		
    		switch(edgeID){
    		case MotionEvent.EDGE_LEFT: edge = "Edge_Left";
    		case MotionEvent.EDGE_BOTTOM: edge = "Edge_Bottom";
    		case MotionEvent.EDGE_RIGHT: edge = "Edge_Right";
    		case MotionEvent.EDGE_TOP: edge = "Edge_Top";
    		
    		}
    		
    		//Hier die Daten, wo und wie gedrckt wurde:
    		button.setText(
    				  "Motion x: " + event.getX() + "[Precision: " + Float.toString(event.getX()) +"]" + newLine
    				+ "Raw x:" + event.getRawX() + newLine
    				+ "Motion y: " + event.getY() + "[Precision: " + event.getYPrecision() +"]" + newLine
    				+ "Raw y:" + event.getRawY() + newLine
    				+ "Motion Pressure: " + event.getPressure() + newLine
    				+ "Edge: " + edge
    				);
    		return true; //return touched
    		
    	}
    }

}


